function [team, cc_score] = mdAlk(W, g, skill_matrix, lbs)
%
% (C)2012-13 Syama Sundar Rangapuram, Thomas Buehler and Matthias Hein 
% Machine Learning Group, Saarland University, Germany
% http://www.ml.uni-saarland.de

    %[sg, six] = sort(g);
    tol_score = 1e-6;
    W_orig = W;
    n = size(W,1);
    ixs = 1:n;
    
    D = cell(1);
    D{1} = [];
    
    G = cell(1);
    G{1} = ixs;
    
    H = cell(1);
    H{1} = [];
    
    team_D = cell(1);
    team_D{1} = boolean(sparse(n,1));
    
    team_D_prime = cell(1);
    %team_D_prime{1} = boolean(sparse(n,1));

    scores = [];
    
    i = 1;
    
%     Cs = cell(1,1);
%     W_rem = W;
%     n_rem = size(W_rem,1);
%     
%     i = 1;
%    C_prime = 1:n_rem;

 
    satisfied = false;
    while ~satisfied
        
        %C = max_density_goldberg(W_rem, false);
        
        W_i_ix = W(G{i}, G{i}); % We are using the original full W in each iteration to have easy indexing. ...
                                % However we add self loops to W in each iteration. The shrink operation is
                                % done only at the index level. We do not remove edges from W.
        g_i_ix = g(G{i});
        n_tilde = size(W_i_ix,1);
        
        if n_tilde == 1
            H_i_ix = 1;
            density = W_i_ix(1,1);
        else
            %[H_i_ix, density] = forte(W_i_ix, 0, 0, g_i_ix, sparse(n_tilde, 1), sparse(n_tilde, 1), [], sparse(n_tilde, n_tilde), 0, 1000, 1, 0, tol_score);
            [H_i_ix, density] = densest_subgraph(W_i_ix,  g_i_ix, tol_score);
            assert( abs( density - generalized_density(W_i_ix, g_i_ix, H_i_ix) ) <= 1e-8 ); 
            H_i_ix = H_i_ix>0;
                                %forte_soft_cnstrs(W_i_ix, n_tilde, 1, g, ones(n_tilde,1), ones(n_tilde,1), [], sparse(n_tilde, n_tilde), 0, rand(n_tilde, 1), 1000, 0, 0);
                                %forte_soft_cnstr(W,ubs,lbs,g,H_ubs, H_lbs,J,dist, d_0, start,MAXITER,gamma, norm_penalties);
                                %forte(W,ubs,lbs,g,H_ubs, H_lbs,J,dist, d_0, MAXITER, nRuns, perturbation)
                                            
            disp(['Iteration: ', num2str(i), '  Found a subgraph of denisty ', num2str(density)]);
        end
        %disp(['density in iteration ', num2str(i), ' is ', num2str(density)]);
        clear W_i_ix;
        
        H{i+1} = G{i}(H_i_ix);  % The densest subgrah found in the current iteration.
                                % The indices are with respect to the oringal full graph 1:n.
        D{i+1} = union(D{i}, H{i+1});  % Union of all the densest subgraphs found until the current iteration. 
                                       % The indices are with respect to the oringal full graph 1:n. 
        
        G{i+1} = setdiff(G{i}, H{i+1});  % Shrink the current subgraph by removing the recent densest subgraph.
        self_edges_val = sum(W(G{i+1}, H{i+1}), 2);  % add self loops in the shrunk graph.
        [ix, jx, val] = find(W);
        W = sparse([ix; G{i+1}'], [jx; G{i+1}'], [val; self_edges_val], n, n);
                    % Here we add self loops to the original graph itself without shrinking it, to have easy indexing of vertices.
                    % Note that when a new self loop is added to a vertex
                    % v, then sparse command sums the new self edge weight to the weight of the current self loop of v.
        

        i = i+1;
                
        team_D{i} = boolean(sparse(n, 1));
        team_D{i}(D{i}) = true;
        
        satisfied = sum( team_D{i}'* skill_matrix < lbs ) ==0;
        
    end
    
    
    for i=1:length(D)
       
        %D_i_prime = D{i};
        team_D_prime{i} = team_D{i};        
        k = team_D_prime{i}'*skill_matrix;       
        
        for j=1:size(skill_matrix, 2)
            
            nx = max(0, lbs(j) - k(j));
            
            % add nx new nodes of skill j to team_D_prime
            nodes_skill_j = find(skill_matrix(:, j));            
            %RTM version had this: nodes = setdiff( nodes_skill_j, D{i});
            new_nodes_skill_j = setdiff( nodes_skill_j, D{i});
%             rnodes = nodes(randperm(length(nodes)));
%             team_D_prime{i}(rnodes(1:nx)) = true;

            [sorted_g_new_nodes, sjx] = sort(g(new_nodes_skill_j));  % Small rank means high expertise. So we need ascending order.
            sorted_new_nodes_skill_j = new_nodes_skill_j(sjx(1:nx));  % We need only nx new nodes!
                                                                      % nx smallest elements are present in indices given by sjx(1:nx)  
            %disp(['min rank = ', num2str(min(g(nodes))), ' max rank = ', num2str(max(g(nodes)))', ' sg(1) = ', num2str(nodes(sjx(1)))]);
            team_D_prime{i}(sorted_new_nodes_skill_j) = true;

        end
        
        assert( sum( team_D_prime{i}'* skill_matrix < lbs ) ==0 );
        
        scores(i) = generalized_density(W_orig, g, team_D_prime{i});
        disp(['score in iteration ', num2str(i), ' is ', num2str(scores(i))]);
        
    end
       
    [max_score, max_ix] = max(scores);
    team = full(team_D_prime{max_ix});
    cc_score = scores(max_ix);
    
    assert( cc_score == generalized_density(W_orig, g, team) );  % team is a boolean vector.

end
