function [lambda,SG_lbs,SG_ubs]=objective_forte_soft_cnstrs( ...
                f,dc,gamma,num,lbs,ubs,wval,ix,jx,g,H_lbs,H_ubs,deg_tilde, assocJ, gvolJ, dist, norm_penalties, debug)
%
% (C)2012-13 Syama Sundar Rangapuram, Thomas Buehler and Matthias Hein 
% Machine Learning Group, Saarland University, Germany
% http://www.ml.uni-saarland.de

% g as in vol_g(C) / assoc(C)
% h as in k1 \le vol_h(C) \le k2
% degree is the actual degree of vertices and used in the computation of assoc(C)!

    if ~exist('debug', 'var')
        debug = false;
    end
    [fsort,sortind]=sort(f);
    
%   More infomration about our subgradient implementation in the team_formation.tex    
%     Supergradient for the penalty term of upper bound constraint, vol(C) <= ub : max{0, vol(C) - ub}
%         \begin{align*}
%         \big(\partial P(f)\big)_{i} = \hat{P}(C_{i}) - \hat{P}(C_{i+1})  &=   \left\{\begin{array}{ll} 0 & \vol_{h}(C_{i}) \le k\\
%                                 \vol_{h}(C_{i}) - k& \vol_{h}(C_{i}) > k ,\vol_{h}(C_{i+1}) \leq k\\
%                                 h_{i} & \vol_{h}(C_{i+1}) > k
%                                     \end{array} \right.
%         \end{align*}

    
    nubs = length(ubs);
    SG_ubs = zeros(num, nubs);
    
    for i=1:nubs
        h = H_ubs(:, i);
        shvol = h(sortind);
        sg = zeros(num,1);
        cumvols = sum(shvol) - [0; cumsum(shvol(1:num-1))];
        l = find(cumvols > ubs(i), 1, 'last');

        if ~isempty(l) % otherwise the subgradient is zero.
            sg(sortind(1:l-1)) = shvol(1:l-1);
            if l~=num
                sg(sortind(l+1:num)) = 0;
            end
            sg(sortind(l)) = cumvols(l)-ubs(i);
        end
        
        assert( abs(max(0, sum(h(dc>0)) - ubs(i)) - dc'*sg) <= 1e-6 );
        SG_ubs(:,i) = sg;
    end
    
    
%     Subgradient for the second term of the penalty term of lower bound constraint, vol(C) >= lb : lb - min{lb, vol(C)
%     \begin{align*}
%         \big(\partial P(f)\big)_{i} = \hat{P}(C_{i}) - \hat{P}(C_{i+1})  &=   \left\{\begin{array}{ll} 0 & \vol_{h}(C_{i+1}) > k\\
%                                 k - \vol_{h}(C_{i+1}) & \vol_{h}(C_{i}) > k ,\vol_{h}(C_{i+1}) \le k\\
%                                 h_{i} & \vol_{h}(C_{i}) \le k
%                                     \end{array} \right.
%     \end{align*}
    
    nlbs = length(lbs);
    SG_lbs = zeros(num, nlbs);
    
    for i=1:nlbs
        h = H_lbs(:, i);
        shvol = h(sortind);
        sg = zeros(num,1);
        cumvols = sum(shvol) - [0; cumsum(shvol(1:num-1))];
        l = find(cumvols > lbs(i), 1, 'last');
        if isempty(l)
            sg = h;
        else
            if l < num
                sg(sortind(1:l-1)) = 0;
                sg(sortind(l+1:num)) = shvol(l+1:num);
                sg(sortind(l)) = lbs(i) - cumvols(l+1);
            else if l==num
                    sg(sortind(num)) = lbs(i); % since cumvols(num+1) = 0, since it is empty set.
                end
            end
            
        end   
        
        assert( abs(min(lbs(i), sum(h(dc>0))) - dc'*sg) <= 1e-6 );
        SG_lbs(:,i) = sg;
    end        
%     assert( min(k1, sum(h(dc>0))) == dc'*sg1 );
%     assert( max(0, sum(h(dc>0)) - k2) == dc'*sg2 );

    
    
    [dix, djx, dval] = find(dist);
    
    penalty_term = norm_penalties'* [max(f)*lbs - f'*SG_lbs f'*SG_ubs -0.5*dval'*abs(f(dix)-f(djx))]';
    
    lambda= (f'*(g+gamma*norm_penalties(end)*sum(dist,2)) + gvolJ*max(f) + gamma*penalty_term)...
                        /(deg_tilde'*f - 0.5*sum(wval.*abs(f(ix)-f(jx))) + assocJ*max(f));
                    
%     lambda= (f'*(g+gamma*2*sum(dist,2)) + gvolJ*max(f) + gamma*sum(f'*SG_ubs) -gamma*sum(f'*SG_lbs)  ... 
%                 + gamma*sum(lbs)*max(f) - gamma*sum(dval.*abs(f(dix)-f(djx)))) ...
%                         /(deg_tilde'*f - 0.5*sum(wval.*abs(f(ix)-f(jx))) + assocJ*max(f));
    
    penalty_term_dc = norm_penalties'* [max(dc)*lbs - dc'*SG_lbs dc'*SG_ubs -0.5*dval'*abs(dc(dix)-dc(djx))]';
    lambda_dc = (dc'*(g+gamma*norm_penalties(end)*sum(dist,2)) + gvolJ*max(dc) + gamma*penalty_term_dc) ...
                        /(deg_tilde'*dc - 0.5*sum(wval.*abs(dc(ix)-dc(jx))) + assocJ*max(dc));
                    
%     lambda_dc = (dc'*(g+gamma*2*sum(dist,2)) + gvolJ*max(dc) + gamma*sum(dc'*SG_ubs) -gamma*sum(dc'*SG_lbs) ... 
%                 + gamma*sum(lbs)*max(dc) - gamma*sum(dval.*abs(dc(dix)-dc(djx)))) ...
%                         /(deg_tilde'*dc - 0.5*sum(wval.*abs(dc(ix)-dc(jx))) + assocJ*max(dc));
%    assert(lambda_dc <= lambda);
    if (lambda_dc - lambda) > 1e-6 && debug
        assert(false, ['Opt thresh seems not to produce descent - lambda: ', num2str(lambda), ' and lambda_dc: ', num2str(lambda_dc)]);
    end

end