%
% (C)2012-13 Syama Sundar Rangapuram, Thomas Buehler and Matthias Hein 
% Machine Learning Group, Saarland University, Germany
% http://www.ml.uni-saarland.de

% M denotes the skill matrix
% Here is the skill order: [ DB DM AI T ]
% Set initialization here: (To get the LP bound set initialization = 2)

%initialization = 0; % No initialization
%initialization = 1; % Initialize with the team found by mdAlk; result of mdAlk is also saved here!
%initialization = 2; % Initialize with the team found from LP relaxation; result of LP relaxation is also saved here!
%tasks = [5]; % Choose the task here. You can choose more than one task: tasks = 1:9 will run the code for all tasks.

task = input('Enter the task number (1-9): ');
initialization = input('Enter the initialization option for Forte (0-2): '); 

load dataForQualitativeExperiments

options.MAXITER = 1000;
options.perturbation = true;
options.tol_score = 1e-4;

display(['Task: ', num2str(task)]);
switch task     % skill order: [ DB DM AI T ]

    case 1, % Task 1: unconstrained
            lbs = 0; M_lbs = sparse(n,1); ubs = 0; M_ubs = sparse(n,1); S = []; dist = sparse(n, n); d_0 = 0;

    case 2, % Task 2: DB >= 3
            lbs = 3; M_lbs = M(:, 1); ubs = 0; M_ubs = sparse(n,1); S = []; dist = sparse(n,n); d_0 = 0;    

    case 3, % Task 3: AI >= 4
            lbs = 4; M_lbs = M(:, 3); ubs = 0; M_ubs = sparse(n,1); S = []; dist = sparse(n,n); d_0 = 0;    

    case 4, % Task 4: AI >=4, d_0 = 2, S = A. Ng
            lbs = 4; M_lbs = M(:, 3); ubs = 0; M_ubs = sparse(n,1); S = 5023; load dblp_all_shortest_paths; dist = Dun; clear Dun; d_0 = 2;    % Andrew Y. Ng                                                  

    case 5, % Task 5: AI >=4, d_0 = 2, S = B. Schoelkopf 
            lbs = 4; M_lbs = M(:, 3); ubs = 0; M_ubs = sparse(n,1); S = 711; load dblp_all_shortest_paths; dist = Dun; clear Dun; d_0 = 2;    % Bernhard

    case 6, % Task 6: AI >=4, d_0 = 2, S = B. Schoelkopf, budget <= 255            
            lbs = 4; M_lbs = M(:, 3); ubs = 255; M_ubs = cost; S = 711; load dblp_all_shortest_paths; dist = Dun; clear Dun; d_0 = 2;    % Bernhard    

    case 7, % Task 7: 3 <= DB <= 6, DM >= 10
            lbs = [3 10]; M_lbs = M(:, 1:2); ubs = 6; M_ubs = M(:, 1); S = []; dist = sparse(n,n); d_0 = 0;

    case 8, % Task 8: 2 <= DB <= 5, 10 <= DM <= 15, 5 <= AI <= 10
            lbs = [2 10 5]; M_lbs = M(:, 1:3); ubs = [5 15 10]; M_ubs = M_lbs; S = []; dist = sparse(n,n); d_0 = 0;

    case 9, % Task 9: AI = 2, T >= 2, size <= 6
            lbs = [2 2]; M_lbs = M(:, 3:4); ubs = [2 6]; M_ubs = [M(:, 3) ones(n,1)]; S = []; dist = sparse(n,n); d_0 = 0;

end

switch initialization
    case 0, [team_forte, score_forte, time_forte] = forte(W, M_lbs, lbs, M_ubs, ubs, g, S, dist, d_0, options); 
            ix_forte = find(team_forte);
            display('Team found by Forte: ');
            for i=1:length(ix_forte)
                display(names_skill_comp{ix_forte(i)});                                        
            end
            display(['Score of the team found by Forte: ', num2str(score_forte)]);
            dataset_results = 'qulaitativeExperiments_forte_task_'; 
    case 1, 
            if task <= 3
                [team_mdAlk, score_mdAlk] = mdAlk(W, g, M_lbs, lbs); 

                ix_mdAlk = find(team_mdAlk);
                display('Team found by mdAlk: ');
                for i=1:length(ix_mdAlk)
                    display(names_skill_comp{ix_mdAlk(i)});                                        
                end
                display(['Score of the team found by mdAlk: ', num2str(score_mdAlk)]);

                options.initialization = team_mdAlk;
                [team_forte, score_forte, time_forte] = forte(W, M_lbs, lbs, M_ubs, ubs, g, S, dist, d_0, options);
                ix_forte = find(team_forte);
                display('Team found by Forte: ');
                for i=1:length(ix_forte)
                    display(names_skill_comp{ix_forte(i)});                                        
                end
                display(['Score of the team found by Forte: ', num2str(score_forte)]);
            else
                display('The method mdAlk cannot solve this task!');
            end
            dataset_results = 'qulaitativeExperiments_mdAlk_plus_forte_task_'; 

    case 2, 
            addpath('/opt/ibm/ILOG/CPLEX_Studio124/cplex/matlab/');
            if ~isempty(S) && d_0 > 0 % We have seed and distance constraints, so we solve an equivalent problem on the reduced graph.
                % Experts that are at a distance of larger than d_0 from any expert in S can be eliminated without changing the problem
                ix_reduced = find(sum(dist(:, S)<=d_0, 2)==length(S)); % the distance should be smaller or equal to each of the expert in S.
                S_reduced = find(ismember(ix_reduced, S));      % S_reduced is the index of S in these indices.
                W_reduced = W(ix_reduced, ix_reduced);
                dist_reduced = dist(ix_reduced, ix_reduced);
                g_reduced = g(ix_reduced);
                M_lbs_reduced = M_lbs(ix_reduced, :);
                M_ubs_reduced = M_ubs(ix_reduced, :); 

                [y, X, f, alpha, tvar, objective, status, status_text, lp_time, lp_set_time] = gdsp_lp_relax_cplex(W_reduced,ubs',lbs',g_reduced, M_ubs_reduced, M_lbs_reduced, S_reduced, dist_reduced, d_0);                
                %team_lp_dist = opt_thresh_forte_hard_cnstrs(W_dist, y_lp_dist, g_dist, lbs, ubs, H_lbs_dist, H_ubs_dist, sum(sum(W(J_dist,J_dist))), sum(g(J_dist)), W_dist(ind_rest_dist, J_dist), dist_dist>0);
                dist_reduced(dist_reduced<=d_0) = 0;
                team_lp_reduced = opt_thresh_forte_hard_cnstrs(W_reduced, y, g_reduced, lbs, ubs, M_lbs_reduced, M_ubs_reduced, sum(sum(W(S,S))), sum(g(S)), W_reduced(:, S_reduced), dist_reduced>0);

                n = size(W,1);
                team_lp = sparse(n,1);
                team_lp(ix_reduced(team_lp_reduced>0)) = 1; 
            else
                [y, X, f, alpha, tvar, objective, status, status_text, lp_time, lp_set_time] = gdsp_lp_relax_cplex(W,ubs',lbs',g, M_ubs, M_lbs, S, dist, d_0);                
                %team_lp = opt_thresh_forte_hard_cnstrs(W, y_lp, g, lbs, ubs, H_lbs, H_ubs, sum(sum(W(S,S))), sum(g(S)), W(ind_rest, S), dist>0);
                dist(dist<=d_0) = 0;
                team_lp = opt_thresh_forte_hard_cnstrs(W, y, g, lbs, ubs, M_lbs, M_ubs, sum(sum(W(S,S))), sum(g(S)), W(:, S), dist>0);
            end                                


            score_lp = generalized_density(W, g, team_lp);

            display(['Lower bound found by LP relaxation: ', num2str(objective)]);

            if isnan(score_lp)
                display(['Team satifsying the given constraints cannot be obtained from the solution of LP relaxation via thresholding']);
            else
                ix_lp = find(team_lp);
                display('Team found from LP relaxation: ');
                for i=1:length(ix_lp)
                    display(names_skill_comp{ix_lp(i)});                                        
                end
                display(['Score of the team found from LP relaxation: ', num2str(score_lp)]);

                options.initialization = team_lp;
                [team_forte, score_forte, time_forte]  = forte(W, M_lbs, lbs, M_ubs, ubs, g, S, dist, d_0, options);                                    
                ix_forte = find(team_forte);
                display('Team found by Forte: ');
                for i=1:length(ix_forte)
                    display(names_skill_comp{ix_forte(i)});                                        
                end
                display(['Score of the team found by Forte: ', num2str(score_forte)]);
            end

            dataset_results = 'qulaitativeExperiments_lp_plus_forte_task_'; 
end

clear dist;
dataset_results = [dataset_results num2str(task)];
save(dataset_results);
